@@ -1,4 +1,5 @@
 import inspect
+import functools
 import re
 import traceback
 import typing as ty
@@ -24,6 +25,23 @@ NESTED_NONE = 'none'
 
 ANSI_ESC_SEQ_RE = re.compile(r'\x1B\[\d+(;\d+){0,2}m', flags=re.MULTILINE)
 
+_T_Formatter = ty.Callable[[click.Context], ty.Generator[str, None, None]]
+
+
+def _process_lines(event_name: str) -> ty.Callable[[_T_Formatter], _T_Formatter]:
+    def decorator(func: _T_Formatter) -> _T_Formatter:
+        @functools.wraps(func)
+        def process_lines(ctx: click.Context) -> ty.Generator[str, None, None]:
+            lines = list(func(ctx))
+            if "sphinx-click-env" in ctx.meta:
+                ctx.meta["sphinx-click-env"].app.events.emit(event_name, ctx, lines)
+            for line in lines:
+                yield line
+
+        return process_lines
+
+    return decorator
+
 
 def _indent(text: str, level: int = 1) -> str:
     prefix = ' ' * (4 * level)
@@ -123,6 +141,7 @@ def _format_help(help_string: str) -> ty.Generator[str, None, None]:
     yield ''
 
 
+@_process_lines("sphinx-click-process-description")
 def _format_description(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the description for a given `click.Command`.
 
@@ -134,6 +153,7 @@ def _format_description(ctx: click.Context) -> ty.Generator[str, None, None]:
         yield from _format_help(help_string)
 
 
+@_process_lines("sphinx-click-process-usage")
 def _format_usage(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the usage for a `click.Command`."""
     yield '.. code-block:: shell'
@@ -163,6 +183,7 @@ def _format_option(opt: click.Option) -> ty.Generator[str, None, None]:
             yield _indent(line)
 
 
+@_process_lines("sphinx-click-process-options")
 def _format_options(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all `click.Option` for a `click.Command`."""
     # the hidden attribute is part of click 7.x only hence use of getattr
@@ -189,6 +210,7 @@ def _format_argument(arg: click.Argument) -> ty.Generator[str, None, None]:
     )
 
 
+@_process_lines("sphinx-click-process-arguments")
 def _format_arguments(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all `click.Argument` for a `click.Command`."""
     params = [x for x in ctx.command.params if isinstance(x, click.Argument)]
@@ -216,6 +238,7 @@ def _format_envvar(
     yield _indent('Provide a default for :option:`{}`'.format(param_ref))
 
 
+@_process_lines("sphinx-click-process-envars")
 def _format_envvars(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format all envvars for a `click.Command`."""
 
@@ -255,6 +278,7 @@ def _format_subcommand(command: click.Command) -> ty.Generator[str, None, None]:
             yield _indent(line)
 
 
+@_process_lines("sphinx-click-process-epilog")
 def _format_epilog(ctx: click.Context) -> ty.Generator[str, None, None]:
     """Format the epilog for a given `click.Command`.
 
@@ -379,7 +403,6 @@ def nested(argument: ty.Optional[str]) -> ty.Optional[str]:
 
 
 class ClickDirective(rst.Directive):
-
     has_content = False
     required_arguments = 1
     option_spec = {
@@ -467,6 +490,7 @@ class ClickDirective(rst.Directive):
         source_name = ctx.command_path
         result = statemachine.ViewList()
 
+        ctx.meta["sphinx-click-env"] = self.env
         if semantic_group:
             lines = _format_description(ctx)
         else:
@@ -540,6 +564,13 @@ class ClickDirective(rst.Directive):
 def setup(app: application.Sphinx) -> ty.Dict[str, ty.Any]:
     app.add_directive('click', ClickDirective)
 
+    app.add_event("sphinx-click-process-description")
+    app.add_event("sphinx-click-process-usage")
+    app.add_event("sphinx-click-process-options")
+    app.add_event("sphinx-click-process-arguments")
+    app.add_event("sphinx-click-process-envvars")
+    app.add_event("sphinx-click-process-epilog")
+
     return {
         'parallel_read_safe': True,
         'parallel_write_safe': True,
